#ifndef _CHATDATAINTERFACE_H__
#define _CHATDATAINTERFACE_H__

#include <list>
#include <map>
#include <string>

namespace Dreamworld
{

class ChatFilterMessage
{
public:

	enum FilterResult
	{
		FR_Waiting = 0,
		FR_Good    = 1,
		FR_Bad     = 2,
		FR_Error   = 3
	};
	
	ChatFilterMessage() :
		m_Result(FR_Waiting),
		m_nRef(1)
	{
	};
	
	~ChatFilterMessage() {};

	// Returns the number of references.
	unsigned int GetNumReferences() 
	{ 
		return m_nRef; 
	};
	
	// Acquire a reference to the object.
	void Acquire() 
	{
		++m_nRef;
	};

	// Release a reference. If we're the last one, we delete ourselves.
	void Release() 
	{
		--m_nRef;		
		// no need for mutex if we're the only one left
		if( m_nRef == 0 )
			delete this;
	};

	std::string m_strFromName;
	std::string m_strToName;
	int m_nTargetType;	
	std::string m_strText;
	char m_cStatus;	
	
	FilterResult m_Result;
	
private:
	// Reference count.
	unsigned int m_nRef;
	
};

class ChatDataInterface
{
public:
	typedef std::list<ChatFilterMessage*> MsgList_t;
	typedef std::map<ChatFilterMessage*,int> HashMsgList_t;
	
	virtual void AddMessage( ChatFilterMessage *pMsg ) = 0;
	virtual ChatFilterMessage *GetMessage() = 0;
	virtual void AddFilteredMessage(ChatFilterMessage *pMsg) = 0;
	virtual void GetFilteredMessages(MsgList_t &filtered) = 0;
};

}

#endif 
