#include "ChatFilter.h"
#include "ChatFilterThread.h"

Dreamworld::ChatFilter::MsgList_t           Dreamworld::ChatFilter::m_MsgList;
Dreamworld::ChatFilter::MsgList_t           Dreamworld::ChatFilter::m_MsgListFiltered;

Dreamworld::ChatFilter::ChatFilter()
{
	curl_global_init(CURL_GLOBAL_DEFAULT);  // this is not thread safe! - but since the constructors are called from main thread, thats fine	
	
	m_pChatFilterThread = new ChatFilterThread(this);	
}

Dreamworld::ChatFilter::~ChatFilter()
{
	if( m_pChatFilterThread != 0 )
	{
		delete m_pChatFilterThread;
		m_pChatFilterThread = 0;
	}
}

void Dreamworld::ChatFilter::AddMessage( ChatFilterMessage *pMsg )
{	
	pMsg->Acquire();
	m_MsgList.push_back(pMsg);
}

void Dreamworld::ChatFilter::AddFilteredMessage( ChatFilterMessage *pMsg )
{	
	pMsg->Acquire();
	m_MsgListFiltered.push_back(pMsg);	
}

Dreamworld::ChatFilterMessage *Dreamworld::ChatFilter::GetMessage()
{	
	ChatFilterMessage *pMsg = 0;
	// TODO: not sure if I have to check for size prior to calling front. doc does not specify		
	if( m_MsgList.size() > 0 )
	{
		// dont release msg, we're giving the reference away so caller is responsible for it
		pMsg = m_MsgList.front();
		m_MsgList.pop_front();
	}
	return pMsg;
}

void Dreamworld::ChatFilter::GetFilteredMessages(MsgList_t &filtered)
{	
	MsgList_t::iterator iter_cur = m_MsgListFiltered.begin();
	MsgList_t::iterator iter_end = m_MsgListFiltered.end();
	for( ; iter_cur != iter_end; ++iter_cur )
	{
		// dont release msg, we're giving the reference away so caller is responsible for it
		filtered.push_back(*iter_cur);
	}
	m_MsgListFiltered.clear();		
}

void Dreamworld::ChatFilter::Clear()
{
	{		
		MsgList_t::iterator iter_cur = m_MsgList.begin();
		MsgList_t::iterator iter_end = m_MsgList.end();
		for( ; iter_cur != iter_end; ++iter_cur )
			(*iter_cur)->Release();
		m_MsgList.clear();
	}
	
	{		
		MsgList_t::iterator iter_cur = m_MsgListFiltered.begin();
		MsgList_t::iterator iter_end = m_MsgListFiltered.end();
		for( ; iter_cur != iter_end; ++iter_cur )
			(*iter_cur)->Release();
		m_MsgListFiltered.clear();
	}
}

void Dreamworld::ChatFilter::Update()
{
	if( m_pChatFilterThread != 0 )
	{
		m_pChatFilterThread->Run();
	}	
	
	// get all the filtered messages and clear the list as fast as possible to let the receive continue	
	ChatFilter::MsgList_t MsgListToProcess;	
	GetFilteredMessages(MsgListToProcess);
	
	// send the messages depending on results from chat filter
	if( MsgListToProcess.size() > 0 )
	{
		ChatFilterMessage *pMsg = 0;
		ChatFilter::MsgList_t::iterator iter_cur = MsgListToProcess.begin();
		ChatFilter::MsgList_t::iterator iter_end = MsgListToProcess.end();
		for( ; iter_cur != iter_end; ++iter_cur )
		{		
			// NOTE: we're not using the provided pointers since we don't know if they are still valid.
			pMsg = *iter_cur;
			
			if( pMsg != 0 )
			{
				switch( pMsg->m_Result )
				{
				case ChatFilterMessage::FR_Good:
					printf("******* Good : %s.\n\r", pMsg->m_strText.c_str());
					break;
				case ChatFilterMessage::FR_Bad:
					printf("******* Bad  : %s.\n\r", pMsg->m_strText.c_str());
					break;
				default:
				case ChatFilterMessage::FR_Error:
					printf("******* Error: %s.\n\r", pMsg->m_strText.c_str());
					break;
				};
									
				// reference was ours so make sure to release the message
				pMsg->Release();			
				pMsg = 0;
			}
		}
		MsgListToProcess.clear();	
	}
}

