#include "ChatFilterThread.h"
#include "ChatDataInterface.h"
#include <vector>

std::string Dreamworld::ChatFilterThread::SEPARATOR_TAG         = ":";
std::string Dreamworld::ChatFilterThread::GROUP_MESSAGE_TAG     = "65";
std::string Dreamworld::ChatFilterThread::PRIVATE_MESSAGE_TAG   = "30";
std::string Dreamworld::ChatFilterThread::USER_TAG              = "c";
std::string Dreamworld::ChatFilterThread::MODERATOR_MESSAGE_TAG = "85";
std::string Dreamworld::ChatFilterThread::DEFAULT_LANGUAGE_TAG  = "en";

std::string Dreamworld::ChatFilterThread::m_WatchDogEmail        = "adamg@funcom.com";
std::string Dreamworld::ChatFilterThread::m_SMTP                 = "smtp://smtp.funcom.com:587";
std::string Dreamworld::ChatFilterThread::m_SMTPUserName         = "";
std::string Dreamworld::ChatFilterThread::m_SMTPPassword         = "";
std::string Dreamworld::ChatFilterThread::m_SMTPMailFrom         = "";
std::string Dreamworld::ChatFilterThread::m_SMTPMailRecipient    = "";

size_t      Dreamworld::ChatFilterThread::m_ChatFilterStringMaxLength = 256;
bool        Dreamworld::ChatFilterThread::m_bWatchDogTriggered        = false;
std::string Dreamworld::ChatFilterThread::m_ChatFilterAddress         = "http://purify.funcom.com/check";
int         Dreamworld::ChatFilterThread::m_ChatFilterPort            = 80;

std::string Dreamworld::ChatFilterThread::RESULT_CODE_TAG             = "\"result-code\": ";
std::string Dreamworld::ChatFilterThread::BAD_WORDS_TAG               = "\"badwords\": ";

static size_t TestConnectionCallback(void *contents, size_t size, size_t nmemb, void *userp);
static size_t MemoryCallback(void *contents, size_t size, size_t nmemb, void *userp);

Dreamworld::ChatFilterThread::ChatFilterThread(Dreamworld::ChatDataInterface* pOwner) :
	m_nIterationTimeout(100000000), // NANOSEC  (100 milliseconds) (arbitrary value)
	m_nIterationSleep(0),        // MICROSEC (0 milliseconds) (arbitrary value)
	m_bServerAvailable(false),
	m_bCannotConnectMsg(false),	
	m_pOwner(pOwner),
	m_pConnectionTest(0)	
{
}

Dreamworld::ChatFilterThread::~ChatFilterThread()
{
	m_pOwner = 0;
}

void Dreamworld::ChatFilterThread::Run()
{
	if( m_pOwner == 0 )
		return;
	// if we're not connected, try to connect now
	if( !m_bServerAvailable )
	{		
		UpdateMessageConnection();
	}
	else
	{
		UpdateMessageRequests();			
		UpdateMessageProcessing();
	}
}

void Dreamworld::ChatFilterThread::UpdateMessageConnection()
{

	// Use a time out instead
	if( !m_bServerAvailable && m_pConnectionTest == 0 )
	{		
		CURL  *pCurlE = curl_easy_init();
		CURLM *pCurlM = curl_multi_init();
		curl_easy_setopt(pCurlE, CURLOPT_PROTOCOLS, CURLPROTO_HTTP);
		curl_easy_setopt(pCurlE, CURLOPT_URL, Dreamworld::ChatFilterThread::m_ChatFilterAddress.c_str());
		curl_easy_setopt(pCurlE, CURLOPT_NOPROGRESS, 1);
		curl_easy_setopt(pCurlE, CURLOPT_WRITEFUNCTION, TestConnectionCallback);
		/*CURLMcode res = */curl_multi_add_handle(pCurlM, pCurlE);
		m_pConnectionTest = new FilterRequest(0, m_pOwner, pCurlM, pCurlE, 0);
		curl_easy_setopt(pCurlE, CURLOPT_WRITEDATA, 0);
	}
	
	if( m_pConnectionTest == 0 )
		return;
	
	CURLMcode res = curl_multi_perform(m_pConnectionTest->m_pCurlM, &m_pConnectionTest->m_nRunningHandles);
	
	int nMsgInQueue = 0;
	CURLMsg *pMessage = curl_multi_info_read(m_pConnectionTest->m_pCurlM, &nMsgInQueue); 
	
	if( (res == CURLM_OK) && pMessage != 0 && (pMessage->msg == CURLMSG_DONE) )
	{
		switch( pMessage->data.result )
		{
		case CURLE_OK:
			m_bCannotConnectMsg = false;
			m_bServerAvailable = true;
			printf("Chat filter available - '%s'.\n\r", Dreamworld::ChatFilterThread::m_ChatFilterAddress.c_str());
			break;				
		default:
			printf("CURL: Cannot connect to chat filter %s (%s).\n\r", Dreamworld::ChatFilterThread::m_ChatFilterAddress.c_str(),
				curl_easy_strerror(pMessage->data.result) );
			break;								
		}
		
		curl_multi_cleanup(m_pConnectionTest->m_pCurlM);
		curl_easy_cleanup(m_pConnectionTest->m_pCurlE);
		delete m_pConnectionTest;
		m_pConnectionTest = 0;		
	}
}

void Dreamworld::ChatFilterThread::UpdateMessageRequests()
{
	ChatFilterMessage *pMsg = 0;
	//time_t startTime = 0;
	//startTime = ServerInstance->Time_ns(); // this function is thread safe.
	//while( (ServerInstance->Time_ns() - startTime) < m_nIterationTimeout )
	while( true )
	{
		pMsg = m_pOwner->GetMessage(); // msg is still acquired but not in the list anymore
		if( pMsg == 0 )
			break;
		else
		{
			// refence is ours now - make sure to release when not needed anymore
			RequestFiltering(pMsg);
			pMsg->Release();
			pMsg = 0;
		}
	}
}

void Dreamworld::ChatFilterThread::UpdateMessageProcessing()
{		
	std::vector<RequestsList_t::iterator> toDelete;
	
	{
		RequestsList_t::iterator iter_cur = m_RequestsList.begin();
		RequestsList_t::iterator iter_end = m_RequestsList.end();			
		for( ; iter_cur != iter_end; ++iter_cur )
		{
			FilterRequest &filterRequest = *iter_cur;
			
			/*CURLMcode res = */curl_multi_perform(filterRequest.m_pCurlM, &filterRequest.m_nRunningHandles);
			
//			if( res == CURLM_CALL_MULTI_PERFORM  )
//				continue;
			
			// read all messages for this handle
			int nMsgInQueue = 0;
			CURLMsg *pMessage = curl_multi_info_read(filterRequest.m_pCurlM, &nMsgInQueue); 
			while( pMessage != 0 )
			{
				switch( pMessage->msg )
				{
				case CURLMSG_DONE: // thats the only message type defined in the doc
					switch( pMessage->data.result )
					{
					case CURLE_OK:
						// its done! running handles is 0 so no need to do anything else
						break;
					default:
						// error in process - just cancel it
						printf("Curl error processing the filter request! ('%s') - '%s'.\n\r", 
							curl_easy_strerror(pMessage->data.result),							
							filterRequest.m_pMsg->m_strText.c_str());
						filterRequest.m_nRunningHandles = 0;
						break;
					};
					
					break;
				default:
					// doc says thats impossible
					printf("Something impossible happened! (%d)\n\r", (int)pMessage->msg);
					break;
				};				
				pMessage = curl_multi_info_read(filterRequest.m_pCurlM, &nMsgInQueue);
			}

			// all messages were received. just read them and delete this one
			if( filterRequest.m_nRunningHandles == 0 )
			{
				toDelete.push_back(iter_cur);
			}
		}
	}
	
	{
		std::vector<RequestsList_t::iterator>::iterator iter_cur = toDelete.begin();
		std::vector<RequestsList_t::iterator>::iterator iter_end = toDelete.end();		
		for( ; iter_cur != iter_end; ++iter_cur )
		{
			FilterRequest &filterRequest = *(*iter_cur);
			curl_multi_cleanup(filterRequest.m_pCurlM);
			curl_easy_cleanup(filterRequest.m_pCurlE);				
			ProcessFilteredMessage(filterRequest);			
			m_RequestsList.erase(*iter_cur);			
		}
	}
}

bool Dreamworld::ChatFilterThread::ProcessFilteredMessage(FilterRequest &filterRequest)
{
	if( filterRequest.m_pMsg != 0 )
	{
		// error from the chat filter - gets overwritten if we can find a response 
		ChatFilterMessage *pMsg = filterRequest.m_pMsg;
		pMsg->m_Result = Dreamworld::ChatFilterMessage::FR_Error;

		if( filterRequest.m_pMem->size > 0 )
		{
			std::string result = filterRequest.m_pMem->memory;
			// parse the string for parameters
			size_t nResultCodePos = result.find(Dreamworld::ChatFilterThread::RESULT_CODE_TAG);
			if( nResultCodePos != std::string::npos )
			{	
				// 1 or 0 -> +1 refers to size of that 1 or 0
				if( result.size() >= nResultCodePos + Dreamworld::ChatFilterThread::RESULT_CODE_TAG.size() + 1 )
				{
					switch( result.at(nResultCodePos + Dreamworld::ChatFilterThread::RESULT_CODE_TAG.size()) )
					{
					case '0': pMsg->m_Result = Dreamworld::ChatFilterMessage::FR_Good; break;
					case '1': pMsg->m_Result = Dreamworld::ChatFilterMessage::FR_Bad; break;
					};
				}
			}

			filterRequest.m_pOwner->AddFilteredMessage(pMsg);
			return true;
		}
	}
	
	return false;
}

bool Dreamworld::ChatFilterThread::RequestFiltering(ChatFilterMessage *pMsg)
{
	if( pMsg != 0 )
	{
		// remove the encoding text to prevent useless filtering
		std::string strLanguage = "en";
		std::string strCleanedText = pMsg->m_strText;
		if( CleanText( strCleanedText, strLanguage ) && strCleanedText.size() <= m_ChatFilterStringMaxLength )
		{
			m_RequestsList.push_back(FilterRequest(pMsg, m_pOwner, curl_multi_init(), curl_easy_init(), 0));

			FilterRequest &filterRequest = m_RequestsList.back();
			
			filterRequest.m_pMem = (MemoryStruct*) malloc(sizeof(MemoryStruct));
			filterRequest.m_pMem->memory = (char*)malloc(1);
			filterRequest.m_pMem->size = 0;
			
			filterRequest.m_strAddress = m_ChatFilterAddress;
			//filterRequest.m_strAddress += "?language=";
			//filterRequest.m_strAddress += strLanguage;
			//filterRequest.m_strAddress += "&text=";
			filterRequest.m_strAddress += "?text=";
			filterRequest.m_strAddress += strCleanedText;								

			curl_easy_setopt(filterRequest.m_pCurlE, CURLOPT_PROTOCOLS, CURLPROTO_HTTP);
			curl_easy_setopt(filterRequest.m_pCurlE, CURLOPT_URL, filterRequest.m_strAddress.c_str());
			curl_easy_setopt(filterRequest.m_pCurlE, CURLOPT_NOPROGRESS, 1);
			curl_easy_setopt(filterRequest.m_pCurlE, CURLOPT_WRITEFUNCTION, MemoryCallback);
			curl_easy_setopt(filterRequest.m_pCurlE, CURLOPT_WRITEDATA, filterRequest.m_pMem);			
			/*CURLMcode res = */curl_multi_add_handle(filterRequest.m_pCurlM, filterRequest.m_pCurlE);						

			return true;
		}
	}
	
	printf("Cannot process chat message.\n\r");
	pMsg->m_Result = ChatFilterMessage::FR_Error;
	m_pOwner->AddFilteredMessage(pMsg);
	
	return false;
}

void Dreamworld::ChatFilterThread::Replace( std::string &strInput, const std::string strFind, const std::string strReplace )
{	
	size_t pos = 0;
	while((pos = strInput.find(strFind, pos)) != std::string::npos)
	{
		strInput.replace(pos, strFind.length(), strReplace);
		pos += strReplace.length();
	}
}

bool Dreamworld::ChatFilterThread::CleanText( std::string &strText, std::string &strLanguage )
{
	size_t sep = strText.find_first_of(SEPARATOR_TAG);
    if( sep == std::string::npos )
      return false;
	
    std::string type = strText.substr(0,sep);
    std::string data = strText.substr(sep+SEPARATOR_TAG.size());

	if( type.size() <= 0 || data.size() <= 0 )
		return false;
	
	// 65:Name:Text  -- group message
	if( type == GROUP_MESSAGE_TAG )
	{
		sep = data.find_first_of(SEPARATOR_TAG);
		if( sep == std::string::npos )
			return false;
		//std::string name = data.substr(0,sep);
		strText = data.substr(sep+SEPARATOR_TAG.size());		
	}
	//30:83:name#whisper: wazza.
	else if( type == PRIVATE_MESSAGE_TAG )
	{
		sep = data.find_first_of(SEPARATOR_TAG);
		if( sep == std::string::npos )
			return false;
		sep = data.find_first_of(SEPARATOR_TAG,sep+SEPARATOR_TAG.size());
		if( sep == std::string::npos )
			return false;
		//std::string name = data.substr(0,sep);
		strText = data.substr(sep+SEPARATOR_TAG.size());
	}
	else
	{
		// unrecognized string - dont process it
		return false;
	}
	
	// get language
//	sep = data.find_first_of(SEPARATOR_TAG);
//	if( sep == std::string::npos )
//		return false;
//		
//	strLanguage = strText.substr(0, sep);	
//	strText = strText.substr(sep+SEPARATOR_TAG.size());
//	
//	if( strLanguage.size() == 0 || strText.size() == 0 )
//		return false;

	Replace(strText, " ", "%20");
	return true;
}

bool Dreamworld::ChatFilterThread::SendEmail( std::string strSubject, std::string strBody )
{
	return false;
}

static size_t TestConnectionCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
	return size*nmemb;
}

static size_t MemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
	if (!userp)
	{
		return 0;
	}

	size_t realsize = size * nmemb;
	struct Dreamworld::ChatFilterThread::MemoryStruct *mem = (struct Dreamworld::ChatFilterThread::MemoryStruct *)userp;

	mem->memory = (char*)realloc(mem->memory, mem->size + realsize + 1);

	if (mem->memory == 0) 
	{
		return 0;
	}

	memcpy(&(mem->memory[mem->size]), contents, realsize);
	mem->size += realsize;
	mem->memory[mem->size] = 0;

	return realsize;
}


Dreamworld::ChatFilterThread::FilterRequest::FilterRequest(ChatFilterMessage *pMsg, ChatDataInterface *pOwner, CURLM* pM, CURL* pE, MemoryStruct *pMem) : 
	m_pMsg(pMsg), m_pOwner(pOwner), m_pCurlM(pM), m_pCurlE(pE), m_nRunningHandles(1), m_pMem(pMem)
{
	if( m_pMsg != 0 )
		m_pMsg->Acquire();	
}
		
Dreamworld::ChatFilterThread::FilterRequest::~FilterRequest()
{
	if( m_pMsg != 0 )
	{
		m_pMsg->Release();
		m_pMsg = 0;
	}
}
