#ifndef _CHATFILTERTHRED_H__
#define _CHATFILTERTHRED_H__

#include <list>
#include <string>
#include <curl/curl.h> // for some reason, CURL doesn't like forward declarations..

namespace Dreamworld
{

class ChatFilterMessage;
class ChatDataInterface;
class ChatFilterThread
{
public:
	
	struct MemoryStruct
	{
		size_t size;
		char *memory;
	};
	
	class FilterRequest
	{
	public:
		FilterRequest(ChatFilterMessage *pMsg, ChatDataInterface *pOwner, CURLM* pM, CURL* pE, MemoryStruct *pMem);
		~FilterRequest();
		
		ChatFilterMessage *m_pMsg;
		ChatDataInterface *m_pOwner;
		CURLM *m_pCurlM;
		CURL  *m_pCurlE;
		int m_nRunningHandles; // curl takes an int!		
		MemoryStruct *m_pMem;		
		std::string m_strAddress;
	};
	
	typedef std::list<FilterRequest> RequestsList_t;

	ChatFilterThread(ChatDataInterface *pOwner);
	virtual ~ChatFilterThread();
		
	virtual void Run();	
	
	static void Replace( std::string &strInput, const std::string strFind, const std::string strReplace );
	static bool CleanText( std::string &strText, std::string &strLanguage );	
	static bool SendEmail( std::string strSubject, std::string strBody );
	
	bool ChatFilterConnect();	
	bool RequestFiltering(ChatFilterMessage *pMsg);
	bool ProcessFilteredMessage(FilterRequest &pMsgData);
	
	void UpdateMessageConnection();
	void UpdateMessageRequests();
	void UpdateMessageProcessing();
					
	long m_nIterationTimeout;	
	long m_nIterationSleep;			
	bool m_bServerAvailable;
	bool m_bCannotConnectMsg;	
	ChatDataInterface *m_pOwner;	
	RequestsList_t m_RequestsList;
	FilterRequest *m_pConnectionTest;
	
	static std::string m_ChatFilterAddress;
	static int m_ChatFilterPort;
	static std::string m_WatchDogEmail;
	static std::string m_SMTP;
	static std::string m_SMTPUserName;
	static std::string m_SMTPPassword;
	static std::string m_SMTPMailFrom;
	static std::string m_SMTPMailRecipient;
	static bool m_bWatchDogTriggered;
	static size_t m_ChatFilterStringMaxLength;	
	
	static std::string SEPARATOR_TAG;
	static std::string GROUP_MESSAGE_TAG;
	static std::string PRIVATE_MESSAGE_TAG;
	static std::string USER_TAG;
	static std::string MODERATOR_MESSAGE_TAG;
	static std::string DEFAULT_LANGUAGE_TAG;	
	
	static std::string RESULT_CODE_TAG;
	static std::string BAD_WORDS_TAG;	
};

}

#endif 
